@@ -1,6 +1,7 @@
 from abc import ABCMeta, abstractmethod
 from enum import Enum
-from functools import lru_cache
+from functools import lru_cache, reduce
+from typing import Dict, Type, Union
 
 # TYPE
 NONE = 'none'
@@ -31,21 +32,40 @@ EMAIL = 'email'
 UUID = 'uuid'
 REGEX = 'regex'
 
+COND_EXIST = 'cond_exist'
+
 # Wrapper prefix
 _wrapper_splitter = '-'
 _not_prefix = 'not'
 
 
-def not_(check):
+class BaseWrapper:
+    def __init__(self, wrapped_func):
+        self.wrapped_func = wrapped_func
+
+
+class BaseValidator(metaclass=ABCMeta):
+    @staticmethod
+    @abstractmethod
+    def validate(value, extra, data):
+        raise NotImplementedError
+
+
+class UnknownFieldValue:
+    message = 'This field cannot be found in this SPEC'
+
+
+def not_(check) -> str:
     return _not_prefix + _wrapper_splitter + check
 
 
-def get_default_check_2_validator_map():
+def get_default_check_2_validator_map() -> Dict[str, BaseValidator]:
     from data_spec_validator.spec.validators import (
         AmountRangeValidator,
         AmountValidator,
         AnyKeyExistsValidator,
         BoolValidator,
+        CondExistValidator,
         DateRangeValidator,
         DateValidator,
         DecimalPlaceValidator,
@@ -93,10 +113,11 @@ def get_default_check_2_validator_map():
         EMAIL: EmailValidator(),
         UUID: UUIDValidator(),
         REGEX: RegexValidator(),
+        COND_EXIST: CondExistValidator(),
     }
 
 
-def _get_wrapper_cls_map():
+def _get_wrapper_cls_map() -> Dict[str, Type[BaseWrapper]]:
     from .wrappers import NotWrapper
 
     return {
@@ -104,7 +125,7 @@ def _get_wrapper_cls_map():
     }
 
 
-def _get_check_2_validator_map():
+def _get_check_2_validator_map() -> Dict[str, BaseValidator]:
     from .custom_spec.defines import get_custom_check_2_validator_map
 
     default_map = get_default_check_2_validator_map()
@@ -113,7 +134,7 @@ def _get_check_2_validator_map():
     return validator_map
 
 
-def get_validator(check):
+def get_validator(check) -> Union[BaseValidator, BaseWrapper]:
     found_idx = check.find(_wrapper_splitter)
     validator_map = _get_check_2_validator_map()
     ori_validator = validator_map.get(check[found_idx + 1 :], validator_map[DUMMY])
@@ -127,17 +148,10 @@ def get_validator(check):
 
 
 @lru_cache(1)
-def get_unknown_field_value():
+def get_unknown_field_value() -> UnknownFieldValue:
     return UnknownFieldValue()
 
 
-class BaseValidator(metaclass=ABCMeta):
-    @staticmethod
-    @abstractmethod
-    def validate(value, extra, data):
-        raise NotImplementedError
-
-
 class ValidateResult:
     def __init__(self, spec=None, field=None, value=None, check=None, error=None):
         # TODO: Output spec & check information when there's a debug message level for development.
@@ -148,7 +162,7 @@ class ValidateResult:
         self.__error = error
 
     @property
-    def field(self):
+    def field(self) -> str:
         return self.__field
 
     @property
@@ -156,14 +170,10 @@ class ValidateResult:
         return self.__value
 
     @property
-    def error(self):
+    def error(self) -> Exception:
         return self.__error
 
 
-class UnknownFieldValue:
-    message = 'This field cannot be found in this SPEC'
-
-
 class CheckerOP(Enum):
     ALL = 'all'
     # All checks will be validated even when the OP is 'any'
@@ -171,7 +181,7 @@ class CheckerOP(Enum):
 
 
 class Checker:
-    def __init__(self, checks, optional=False, allow_none=False, op=CheckerOP.ALL, extra=None):
+    def __init__(self, checks, optional=False, allow_none=False, op=CheckerOP.ALL, extra=None, **kwargs):
         """
         checks: list of str(Check)
         optional: boolean
@@ -185,28 +195,66 @@ class Checker:
         self._op = op
         self._optional = optional
         self._allow_none = allow_none
-        self.extra = extra or {}
 
-        self._ensure()
+        self._ensure(kwargs)
+        if extra:
+            print('[DSV][WARNING] keyword: extra is gonna be deprecated')
+        deprecating_checks = {KEY_COEXISTS, ANY_KEY_EXISTS}
+        for deprecating_check in deprecating_checks.intersection(set(checks)):
+            print(f'[DSV][WARNING] Check: {deprecating_check.upper()} is gonna be deprecated')
+
+        self.extra = self._merge_extra_kwargs(extra or {}, kwargs)
+
+    @staticmethod
+    def _merge_extra_kwargs(deprecated_extra: dict, check_kwargs: dict) -> dict:
+        extra = deprecated_extra.copy()
+
+        all_keys = set(_get_check_2_validator_map().keys())
+        for arg_k, arg_v in check_kwargs.items():
+            lower_arg_k = arg_k.lower()
+            if lower_arg_k in all_keys:
+                extra[lower_arg_k] = arg_v
+        return extra
+
+    def _ensure(self, check_kwargs):
+        def __ensure_upper_case(_kwargs):
+            non_upper = list(filter(lambda k: not k.isupper(), _kwargs.keys()))
+            if non_upper:
+                raise TypeError(f'Keyword must be upper-cased: {non_upper}')
+
+        def __ensure_no_repeated_forbidden(_kwargs):
+            blacklist = {'optional', 'allow_none', 'op', 'extra'}
+
+            def _check_in_blacklist(acc, key):
+                if key.lower() in blacklist:
+                    acc.add(key)
+                return acc
+
+            forbidden = list(reduce(_check_in_blacklist, _kwargs.keys(), set()))
+            if forbidden:
+                forbidden.sort()
+                raise TypeError(f'Forbidden keyword arguments: {", ".join(forbidden)}')
 
-    def _ensure(self):
         if self._optional and len(self.checks) == 0:
             raise ValueError('Require at least 1 check when set optional to True')
 
+        __ensure_upper_case(check_kwargs)
+        __ensure_no_repeated_forbidden(check_kwargs)
+
     @property
-    def allow_none(self):
+    def allow_none(self) -> bool:
         return self._allow_none
 
     @property
-    def allow_optional(self):
+    def allow_optional(self) -> bool:
         return self._optional
 
     @property
-    def is_op_any(self):
+    def is_op_any(self) -> bool:
         return self._op == CheckerOP.ANY
 
     @property
-    def is_op_all(self):
+    def is_op_all(self) -> bool:
         return self._op == CheckerOP.ALL
 
 
@@ -219,7 +267,7 @@ class MsgLv(Enum):
 __message_level = MsgLv.DEFAULT
 
 
-def get_msg_level():
+def get_msg_level() -> MsgLv:
     return __message_level
 
 
